@@ -1,7 +1,7 @@
 package it.unimi.dsi.fastutil;
 
 /*		 
- * Copyright (C) 2002-2014 Sebastiano Vigna 
+ * Copyright (C) 2002-2016 Sebastiano Vigna
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,10 +16,12 @@ package it.unimi.dsi.fastutil;
  * limitations under the License. 
  */
 
-import java.util.ArrayList;
-
 import it.unimi.dsi.fastutil.ints.IntComparator;
 
+import java.util.ArrayList;
+import java.util.concurrent.ForkJoinPool;
+import java.util.concurrent.RecursiveAction;
+
 /** A class providing static methods and objects that do useful things with arrays.
  *
  * <p>In addition to commodity methods, this class contains {@link Swapper}-based implementations
@@ -71,9 +73,6 @@ public class Arrays {
 		if ( offset + length > arrayLength ) throw new ArrayIndexOutOfBoundsException( "Last index (" + ( offset + length ) + ") is greater than array length (" + arrayLength + ")" );
 	}
 
-	private static final int SMALL = 7;
-	private static final int MEDIUM = 40;
-
 	/**
 	 * Transforms two consecutive sorted ranges into a single sorted range. The initial ranges are
 	 * <code>[first..middle)</code> and <code>[middle..last)</code>, and the resulting range is
@@ -193,7 +192,9 @@ public class Arrays {
 				( bc > 0 ? b : ac > 0 ? c : a ) );
 	}
 
-	/** Sorts the specified range of elements using the specified swapper and according to the order induced by the specified
+	 private static final int MERGESORT_NO_REC = 16;
+
+	 /** Sorts the specified range of elements using the specified swapper and according to the order induced by the specified
 	 * comparator using mergesort.
 	 * 
 	 * <p>This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result
@@ -217,7 +218,7 @@ public class Arrays {
 		final int length = to - from;
 
 		// Insertion sort on smallest arrays
-		if ( length < SMALL ) {
+		if ( length < MERGESORT_NO_REC ) {
 			for ( int i = from; i < to; i++ ) {
 				for ( int j = i; j > from && ( c.compare( j - 1, j ) > 0 ); j-- ) {
 					swapper.swap( j, j - 1 );
@@ -239,13 +240,127 @@ public class Arrays {
 		inPlaceMerge( from, mid, to, c, swapper );
 	}
 
+	/** Swaps two sequences of elements using a provided swapper.
+	 *
+	 * @param swapper the swapper.
+	 * @param a a position in {@code x}.
+	 * @param b another position in {@code x}.
+	 * @param n the number of elements to exchange starting at {@code a} and {@code b}.
+	 */
+	protected static void swap( final Swapper swapper, int a, int b, final int n ) {
+		for ( int i = 0; i < n; i++, a++, b++ ) swapper.swap( a, b );
+	}
+
+	private static final int QUICKSORT_NO_REC = 16;
+	private static final int PARALLEL_QUICKSORT_NO_FORK = 8192;
+	private static final int QUICKSORT_MEDIAN_OF_9 = 128;
+
+	protected static class ForkJoinGenericQuickSort extends RecursiveAction {
+		private static final long serialVersionUID = 1L;
+		private final int from;
+		private final int to;
+		private final IntComparator comp;
+		private final Swapper swapper;
+
+		public ForkJoinGenericQuickSort( final int from, final int to, final IntComparator comp, final Swapper swapper ) {
+			this.from = from;
+			this.to = to;
+			this.comp = comp;
+			this.swapper = swapper;
+		}
+
+		@Override
+		protected void compute() {
+			final int len = to - from;
+			if ( len < PARALLEL_QUICKSORT_NO_FORK ) {
+				quickSort( from, to, comp, swapper );
+				return;
+			}
+			// Choose a partition element, v
+			int m = from + len / 2;
+			int l = from;
+			int n = to - 1;
+			int s = len / 8;
+			l = med3( l, l + s, l + 2 * s, comp );
+			m = med3( m - s, m, m + s, comp );
+			n = med3( n - 2 * s, n - s, n, comp );
+			m = med3( l, m, n, comp );
+			// Establish Invariant: v* (<v)* (>v)* v*
+			int a = from, b = a, c = to - 1, d = c;
+			while ( true ) {
+				int comparison;
+				while ( b <= c && ( ( comparison = comp.compare( b, m ) ) <= 0 ) ) {
+					if ( comparison == 0 ) {
+						// Fix reference to pivot if necessary
+						if ( a == m ) m = b;
+						else if ( b == m ) m = a;
+						swapper.swap( a++, b );
+					}
+					b++;
+				}
+				while ( c >= b && ( ( comparison = comp.compare( c, m ) ) >= 0 ) ) {
+					if ( comparison == 0 ) {
+						// Fix reference to pivot if necessary
+						if ( c == m ) m = d;
+						else if ( d == m ) m = c;
+						swapper.swap( c, d-- );
+					}
+					c--;
+				}
+				if ( b > c ) break;
+				// Fix reference to pivot if necessary
+				if ( b == m ) m = d;
+				else if ( c == m ) m = c;
+				swapper.swap( b++, c-- );
+			}
+
+			// Swap partition elements back to middle
+			s = Math.min( a - from, b - a );
+			swap( swapper, from, b - s, s );
+			s = Math.min( d - c, to - d - 1 );
+			swap( swapper, b, to - s, s );
+
+			// Recursively sort non-partition-elements
+			int t;
+			s = b - a;
+			t = d - c;
+			if ( s > 1 && t > 1 ) invokeAll( new ForkJoinGenericQuickSort( from, from + s, comp, swapper ), new ForkJoinGenericQuickSort( to - t, to, comp, swapper ) );
+			else if ( s > 1 ) invokeAll( new ForkJoinGenericQuickSort( from, from + s, comp, swapper ) );
+			else invokeAll( new ForkJoinGenericQuickSort( to - t, to, comp, swapper ) );
+		}
+	}
+
 	/** Sorts the specified range of elements using the specified swapper and according to the order induced by the specified
-	 * comparator using quicksort. 
+	 * comparator using a parallel quicksort. 
 	 * 
 	 * <p>The sorting algorithm is a tuned quicksort adapted from Jon L. Bentley and M. Douglas
 	 * McIlroy, &ldquo;Engineering a Sort Function&rdquo;, <i>Software: Practice and Experience</i>, 23(11), pages
 	 * 1249&minus;1265, 1993.
 	 * 
+	 * <p>This implementation uses a {@link ForkJoinPool} executor service with {@link Runtime#availableProcessors()} parallel threads.
+	 * 
+	 * @param from the index of the first element (inclusive) to be sorted.
+	 * @param to the index of the last element (exclusive) to be sorted.
+	 * @param comp the comparator to determine the order of the generic data.
+	 * @param swapper an object that knows how to swap the elements at any two positions.
+	 * 
+	 */
+	public static void parallelQuickSort( final int from, final int to, final IntComparator comp, final Swapper swapper ) {
+		final ForkJoinPool pool = new ForkJoinPool( Runtime.getRuntime().availableProcessors() );
+		pool.invoke( new ForkJoinGenericQuickSort( from, to, comp, swapper ) );
+		pool.shutdown();
+	}
+
+
+	/** Sorts the specified range of elements using the specified swapper and according to the order induced by the specified
+	 * comparator using parallel quicksort. 
+	 * 
+	 * <p>The sorting algorithm is a tuned quicksort adapted from Jon L. Bentley and M. Douglas
+	 * McIlroy, &ldquo;Engineering a Sort Function&rdquo;, <i>Software: Practice and Experience</i>, 23(11), pages
+	 * 1249&minus;1265, 1993.
+	 * 
+	 * <p>This implementation uses a {@link ForkJoinPool} executor service with {@link Runtime#availableProcessors()} parallel threads.
+	 * 
 	 * @param from the index of the first element (inclusive) to be sorted.
 	 * @param to the index of the last element (exclusive) to be sorted.
 	 * @param comp the comparator to determine the order of the generic data.
@@ -255,7 +370,7 @@ public class Arrays {
 	public static void quickSort( final int from, final int to, final IntComparator comp, final Swapper swapper ) {
 		final int len = to - from;
 		// Insertion sort on smallest arrays
-		if ( len < SMALL ) {
+		if ( len < QUICKSORT_NO_REC ) {
 			for ( int i = from; i < to; i++ )
 				for ( int j = i; j > from && ( comp.compare( j - 1, j ) > 0 ); j-- ) {
 					swapper.swap( j, j - 1 );
@@ -265,17 +380,15 @@ public class Arrays {
 
 		// Choose a partition element, v
 		int m = from + len / 2; // Small arrays, middle element
-		if ( len > SMALL ) {
-			int l = from;
-			int n = to - 1;
-			if ( len > MEDIUM ) { // Big arrays, pseudomedian of 9
-				int s = len / 8;
-				l = med3( l, l + s, l + 2 * s, comp );
-				m = med3( m - s, m, m + s, comp );
-				n = med3( n - 2 * s, n - s, n, comp );
-			}
-			m = med3( l, m, n, comp ); // Mid-size, med of 3
+		int l = from;
+		int n = to - 1;
+		if ( len > QUICKSORT_MEDIAN_OF_9 ) { // Big arrays, pseudomedian of 9
+			int s = len / 8;
+			l = med3( l, l + s, l + 2 * s, comp );
+			m = med3( m - s, m, m + s, comp );
+			n = med3( n - 2 * s, n - s, n, comp );
 		}
+		m = med3( l, m, n, comp ); // Mid-size, med of 3
 		// int v = x[m];
 
 		int a = from;
@@ -287,44 +400,38 @@ public class Arrays {
 			int comparison;
 			while ( b <= c && ( ( comparison = comp.compare( b, m ) ) <= 0 ) ) {
 				if ( comparison == 0 ) {
-					if ( a == m ) m = b; // moving target; DELTA to JDK !!!
-					else if ( b == m ) m = a; // moving target; DELTA to JDK !!!
+					// Fix reference to pivot if necessary
+					if ( a == m ) m = b;
+					else if ( b == m ) m = a;
 					swapper.swap( a++, b );
 				}
 				b++;
 			}
 			while ( c >= b && ( ( comparison = comp.compare( c, m ) ) >= 0 ) ) {
 				if ( comparison == 0 ) {
-					if ( c == m ) m = d; // moving target; DELTA to JDK !!!
-					else if ( d == m ) m = c; // moving target; DELTA to JDK !!!
+					// Fix reference to pivot if necessary
+					if ( c == m ) m = d;
+					else if ( d == m ) m = c;
 					swapper.swap( c, d-- );
 				}
 				c--;
 			}
 			if ( b > c ) break;
-			if ( b == m ) m = d; // moving target; DELTA to JDK !!!
-			else if ( c == m ) m = c; // moving target; DELTA to JDK !!!
+			// Fix reference to pivot if necessary
+			if ( b == m ) m = d;
+			else if ( c == m ) m = c;
 			swapper.swap( b++, c-- );
 		}
 
 		// Swap partition elements back to middle
 		int s;
-		int n = to;
 		s = Math.min( a - from, b - a );
-		vecSwap( swapper, from, b - s, s );
-		s = Math.min( d - c, n - d - 1 );
-		vecSwap( swapper, b, n - s, s );
+		swap( swapper, from, b - s, s );
+		s = Math.min( d - c, to - d - 1 );
+		swap( swapper, b, to - s, s );
 
 		// Recursively sort non-partition-elements
 		if ( ( s = b - a ) > 1 ) quickSort( from, from + s, comp, swapper );
-		if ( ( s = d - c ) > 1 ) quickSort( n - s, n, comp, swapper );
-	}
-
-
-	/**
-	 * Swaps x[a .. (a+n-1)] with x[b .. (b+n-1)].
-	 */
-	private static void vecSwap( final Swapper swapper, int from, int l, final int s ) {
-		for ( int i = 0; i < s; i++, from++, l++ ) swapper.swap( from, l );
+		if ( ( s = d - c ) > 1 ) quickSort( to - s, to, comp, swapper );
 	}
 }
